# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import List, Optional

from argparse import ArgumentParser
from pathlib import Path


DEFAULT_SRC_ROOT = ...  # type: Path

def make_parser(*, prog: str) -> ArgumentParser: ...

def main(argv: Optional[List[str]] = ...) -> None: ...
