# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import Dict, List, Sequence

from pathlib import Path

class ContentHierarchy:
    root = ...  # type: Path
    def __init__(self, root: Path) -> None: ...
    @property
    def units(self) -> Dict[str, Unit]: ...

class Unit:
    src = ...  # type: Path
    root = ...  # type: ContentHierarchy
    parts = ...  # type: List[Unit]
    def __init__(self, src: Path, *, root: ContentHierarchy) -> None: ...
    @property
    def slug(self) -> str: ...
    def sources(self, *, medium: str, lang: str) -> Sequence[Path]: ...
    def sphinx_extras(self) -> List[str]: ...
